function match_saa_state, roadmap, saa_state=saa_state
;
; boolean - does input match requested saa state?
;  
;    
nout=n_elements(roadmap)

case 1 of
   data_chk(saa_state,/undef): retval=lonarr(nout) + 1     ; dont care (all yes)
   saa_state(0): retval=saa_in(roadmap,/fem_only)
   else: retval=1-(saa_in(roadmap,/fem_only))
endcase   
return,retval
end
  
pro mk_sff_pair,infil,indat=indat,$
	         ssl,ssm,sss,triplet=triplet, $
		 saa_state=saa_state,debug=debug, $
                 al_only=al_only,almg_only=almg_only, $
                 maxdt=maxdt,count=count,delta=delta
;+
;   Name: mk_sff_pair
;
;   Purpose: make pairings for SXT full disk composite movies
;
;   Input Parameters:
;      infil - list of SFR files OR SXT INDEX records 
;         NOTE:  If infil is an INDEX structure then keyword
;         indat=indat must contain the corresponding data array.
;
;   Output Paramters:
;      ssl - subscripts of LONG exposures matching criteria  
;      ssm - subscripts of Medium (if /triplet set)
;      sss - subscripts of associated SHORT exposures 
;      maxdt - maximum deltaT to allow LONG:SHORT - in seconds
;  
;   History:
;      14-apr-1998 - S.L.Freeland - break/rework mk_sfd
;      31-Jan-2000 - S.L.Freeland - enable /TRIPLET
;       7-Nov-2003 - S.L.Freeland - made DELTA = maximum delta (pair or triplet) 
;                                   added MAXDT keyword/function
;      19-nov-2003 - S.L.Freeland - required INDEX, not ROADMAPs
;                                   (CCD Temp check added)
;      13-Jan-2004 - LWA - Removed filta=1 requirement for short and med.
;			   Widened short to 4-15 dpe and med to 16-26 dpe.
;                          Permit warm CCD before 2-oct-91 09:05.
;      19-Jan-2004 - LWA - Lowered required percentd to gt 95.
;                          Increased default maxdt to 660 seconds.
;			   Major rewrite of flow of pairing process.
;      20-Jan-2004 - LWA - Installed test for images with CCD bleed.
;			   For short and medium exposures ignore lost 
;			    data outside of active latitudes (rows 150:405 in HR)
;			    and permit one minor frame of missing data
;			    within the active latitude band.
;      24-Jan-2004 - LWA - Added test for image data completeness of LONG
;			    exposures because percentd not always reliable.
;			   Added test for unsaturated long/medium exposures.
;			    Will pass -1 in ssm or sss to mk_ssc for these cases.
;      26-Jan-2004 - LWA - Debugged spaghetti logic.
;      31-Jan-2004 - LWA - Corrected inconsistency in image counting.
;      25-Mar-2004 - LWA - Fixed bug in short image logic.
;      26-Mar-2004 - LWA - Fixed bug in "nearest short" logic.
;      17-Aug-2004 - LWA - Added keyword indat.
;-

common mk_ssc_blk1, lweek, files, roadmap, xdata

triplet=keyword_set(triplet)
if NOT triplet then ssmcnt=0   ;Needed for if statement later on.
debug=keyword_set(debug)

; ---- Roadmap & xdata either generated internally or ----------------
; ---- obtained from the common statement ----------------------------
; ---- Although called "roadmap" actually index records required -----
; ------- input sfr file list or INDEX -------------------------------
; 
if data_chk(infil,/string) then rd_xda,infil,-1,roadmap,xdata
if data_chk(infil,/struct) then begin
   roadmap=infil
   if n_elements(xdata) eq 0 then xdata=indat
endif   
; -----------------------------------------------
if n_elements(maxdt) eq 0 then maxdts=660 else maxdts=maxdt ;default->11 mins
tim2orbit, roadmap, tim2night=tim2night                 ; get all time2nights
kool='2-oct-91 09:05'    			;Time of CCD cooling to -20 C.
; -------- Define uniq pairing combinations ------
res=  [1,2]            ; half / quarter
case 1 of 
   keyword_set(al_only): fb=2
   keyword_set(almg_only): fb=3
   else: fb=[2,3]
endcase
; --------- initialize output variables ------------
ssl=[-1]                                        ; long subscripts
ssm=[-1]
sss=[-1]                                        ; paired subscripts
; -----------------------------------------------
llow=([16,26])	;llow=([17,26])      LWA 1/13/04
lhi= ([25,30])

for resx=0,n_elements(res)-1 do begin           ; for each resolution
  for filtb=0,n_elements(fb)-1 do begin        ; for each filter 
    ; ----- Test for 1991 pre-CCD-cool images -----
    tc = int2secarr(roadmap,anytim(kool,/yohkoh)) - $
      int2secarr(anytim(kool,/yohkoh))

;    ------ Flag promising LONG exposures ---------
    ssli=where($  ;gt_percentd(roadmap) gt 95  and $
      gt_filtb(roadmap) eq fb(filtb) and $
      gt_filta(roadmap) eq 1 and $
      gt_res(roadmap) eq res(resx) and $
      gt_expmode(roadmap) eq 0 and $
      match_saa_state(roadmap, saa_state=saa_state) and $
      ((gt_temp_ccd(roadmap) lt -20. and tc gt 0) or (tc lt 0)) and $
      gt_dpe(roadmap) ge llow(triplet) and $
      gt_dpe(roadmap) le lhi(triplet) and $
      (gt_shape_cmd(roadmap,/x) eq gt_shape_cmd(roadmap,/y)) and $
      tim2night gt 2.0,sslcnt)
    nnssl=sslcnt
;print,'T1   ','nnssl =',nnssl,'   ','sslcnt  =',sslcnt
    ; ----- Eliminate LONGS with bleed -------------------
    if n_elements(xdata) ne 0 and sslcnt gt 0 then begin
      bleed=sxt_bleedtest(roadmap(ssli),xdata(*,*,ssli))
      bb=where(bleed eq 0,nbb)
      if nbb lt n_elements(ssli) then begin
        ssli=ssli(bb)
        sslcnt=n_elements(ssli)
        nnssl=n_elements(ssli)
      endif   
    endif	 
   ; ---- Eliminate LONGS with missing data > 1 minor frame ----
    if sslcnt gt 0 then begin
      sskp=intarr(sslcnt)
      for sl=0,n_elements(ssli)-1 do begin
        nrow=511/gt_res(roadmap(ssli(sl)))
        ngdok=8/gt_res(roadmap(ssli(sl)))
        datatest=total(xdata(0:nrow,0:nrow,ssli(sl)),1)
        ngd=where(datatest eq 0,nngd)
        if nngd gt ngdok then begin  ;Permit one minor frame missing.
          sskp(sl)=1
          sslcnt=sslcnt-1
          nnssl=nnssl-1
        endif
      endfor
      sskpi=where(sskp eq 0,nsskpi)
      if nsskpi gt 0 then ssli=ssli(sskpi)
    endif
    
;print,'T2   ','nnssl =',nnssl,'   ','sslcnt  =',sslcnt
;stop
;    ------ Cycle through the set of LONG exposures --------
    if sslcnt gt 0 then $
    FOR ll=0,sslcnt-1 do begin
      rdm=roadmap(ssli(ll))    
    ; ---- Check for saturation in LONG exposure ----
    ; ---- sxt_satscreen returns a 1 for unsaturated images ----
      satlong=sxt_satscreen(roadmap(ssli(ll)),xdata(*,*,ssli(ll)))

;print,'T3   ','ll =',ll,'   ','ssli(ll) =',ssli(ll)
;    ------------- MEDIUM exposure (triplet only) --------
      if triplet then begin
        if satlong then begin
          ssmi=-1
          sssi=-1
          ssmcnt=1
          ssscnt=1
          goto,SKIP
        endif
        ssmi=where($  ;gt_percentd(roadmap) gt 85 and $
          gt_filtb(roadmap) eq fb(filtb) and $
          gt_res(roadmap) eq res(resx) and $
          gt_expmode(roadmap) eq 0 and $
          match_saa_state(roadmap, saa_state=saa_state) and $
          ((gt_temp_ccd(roadmap) lt -20. and tc gt 0) or (tc lt 0)) and $
          gt_dpe(roadmap) ge llow(1-triplet) and $
          gt_dpe(roadmap) le lhi(1-triplet) and $
          (gt_shape_cmd(roadmap,/x) eq gt_shape_cmd(roadmap,/y)) and $
          tim2night gt 2.0 and $
          abs(int2secarr(roadmap)-int2secarr(rdm,roadmap(0))) le maxdts, $
          ssmcnt)
;print,'Test 0: ssmi=',ssmcnt
      ; ---- Allow partial images if active latitudes present ----
        if ssmcnt gt 0 then begin
          nrow=511/gt_res(roadmap(ssli(ll)))
          actband=fix(indgen(256)+150)/gt_res(roadmap(ssli(ll)))
          ngdok=8/gt_res(roadmap(ssli(ll)))
          sskp=intarr(ssmcnt)
          for pd=0,ssmcnt-1 do begin
            datatest=total(xdata(0:nrow,0:nrow,ssmi(pd)),1)
            ngd=where(datatest(actband) eq 0,nngd)
            if nngd gt ngdok then begin  ;Permit one minor frame missing.
              sskp(pd)=1		   
              ssmcnt=ssmcnt-1
            endif
          endfor   
          sskpi=where(sskp eq 0,nsskpi)
          if nsskpi gt 0 then ssmi=ssmi(sskpi)
        endif
;print,'Test 1: Missing data',ssmcnt
      ; ---- Select which exposure to use --------------
        if ssmcnt gt 0 then begin
          case 1 of
            ssmcnt gt 1 : begin   ;Choose medium exposure nearest im time.
              ssmdt=tim2dset(roadmap(ssmi),rdm)
              ssmi=ssmi(ssmdt)
            end
            ssmcnt eq 1 : ssmi=ssmi
          endcase  
          ssmcnt=1
        endif
;print,'Test 2: Which exposure',ssmcnt
    ; ----- Eliminate MEDIUMS with bleed -------------------
        if n_elements(xdata) ne 0 and ssmcnt gt 0 then begin
          bleed=sxt_bleedtest(roadmap(ssmi),xdata(*,*,ssmi))
          bb=where(bleed eq 0,nbb)
          if nbb ne 1 then begin
            ssmcnt=0
            ssscnt=0
            nnssl=nnssl-1
          endif
        endif
;print,'Test 3: Bleed check',ssmcnt
    ; ---- Check for saturation in MEDIUM exposure ----
        if ssmcnt gt 0 then begin
          satmed=sxt_satscreen(roadmap(ssmi),xdata(*,*,ssmi))
        endif
        if ssmcnt le 0 then begin
          ssmcnt=0
          ssscnt=0
          nnssl=nnssl-1
        endif	
;print,'Test4: Saturation',ssmcnt,'   ','SATMED=',satmed
      endif

;    ------- SHORT exposures w/matching resolution and filter ----------
      if triplet and ssmcnt ne 0 then begin
        if satlong or satmed then begin
          sssi=-1
          ssmcnt=1
          ssscnt=1
          goto,SKIP
        endif
      endif
      if NOT triplet and satlong then begin	;LONG not saturated
        sssi=-1
        ssscnt=1
        goto,SKIP
      endif
      xx=0
      if NOT triplet then xx=1
      if (triplet and ssmcnt gt 0) then xx=1
      IF xx then begin
        sssi=where(gt_filtb(roadmap) eq fb(filtb) and $
          gt_res(roadmap) eq res(resx) and $
          gt_expmode(roadmap) eq 0 and $
          ((gt_temp_ccd(roadmap) lt -20. and tc gt 0) or (tc lt 0)) and $
          gt_dpe(roadmap) ge 4 and gt_dpe(roadmap) le 15 and $
          (gt_shape_cmd(roadmap,/x) eq gt_shape_cmd(roadmap,/y)) and $
          tim2night gt 2.0 and $
          abs(int2secarr(roadmap)-int2secarr(rdm,roadmap(0))) le maxdts, $
          ssscnt)
;print,'S1   ssscnt =',ssscnt,'   ','nnssl =',nnssl
;help,sssi & print,sssi
        if ssscnt le 0 then begin
		nnssl=nnssl-1
        endif
;print,'S2   ssscnt =',ssscnt,'   ','nnssl =',nnssl
      ; ---- Allow partial images if active latitudes present ----
        if ssscnt gt 0 then begin
          nrow=511/gt_res(roadmap(ssli(ll)))
          actband=fix(indgen(256)+150)/gt_res(roadmap(ssli(ll)))
          ngdok=8/gt_res(roadmap(ssli(ll)))
          sskp=intarr(ssscnt)
          for pd=0,ssscnt-1 do begin
            datatest=total(xdata(0:nrow,0:nrow,sssi(pd)),1)
            ngd=where(datatest(actband) eq 0,nngd)
            if nngd gt ngdok then begin  ;Permit only one minor frame missing.
              sskp(pd)=1
              ssscnt=ssscnt-1
            endif
          endfor
          sskpi=where(sskp eq 0,nsskpi)
          if nsskpi gt 0 then sssi=sssi(sskpi)
          if ssscnt eq 0 then nnssl=nnssl-1	;LWA 31-Jan-2004
        endif
;print,'S3   ssscnt =',ssscnt,'   ','nnssl =',nnssl
;stop
      ; ---- Select which exposure to use --------------
      ; ----  Select the optimum DPE ----
        if ssscnt gt 0 then begin
          if ssscnt gt 1 then begin   ;Choose smallest DPE value.
            sdpe=gt_dpe(roadmap(sssi)) 	
            sd=where(sdpe eq min(sdpe),nsd)
            if nsd gt 1 then begin  ;Choose nearest SHORT exposure.
              ttsd=abs(int2secarr(roadmap(sssi(sd)),roadmap(0)) - $	;LWA debug 3/26/04
              int2secarr(rdm,roadmap(0)))
              ttsdi=where(ttsd eq min(ttsd))
              sd=sd(ttsdi)
            endif	
          endif else begin
            sd=0
          endelse  
          sssi=sssi(sd)
          ssscnt=1
        endif
        if NOT triplet then begin   ;Only worry about bleeds for doublets.
   ; ----- Eliminate SHORTS with bleed -------------------
          if n_elements(xdata) ne 0 and ssscnt gt 0 then begin
            bleed=sxt_bleedtest(roadmap(sssi),xdata(*,*,sssi))
            bb=where(bleed eq 0,nbb)
            if nbb ne 1 then begin
              sssi=-1
              ssscnt=0
              nnssl=nnssl-1
            endif
          endif
        endif
      ENDIF	
      SKIP:
;print,resx,filtb,ll,'   ','SATLONG=',satlong,ssli(ll),sssi
      if (triplet and ssmcnt eq 1 and ssscnt eq 1) then begin
        ssl=[ssl,ssli(ll)]
        ssm=[ssm,ssmi]
        sss=[sss,sssi]
      endif
      if (NOT triplet and ssscnt eq 1) then begin
	ssl=[ssl,ssli(ll)]
        sss=[sss,sssi]
      endif	
;print,'RESX, FILTB, LL, NNSSL',resx,filtb,ll,nnssl
;stop
    ENDFOR
       mess=['#images w/filter:'+gt_filtb(fb(filtb),/string) + $
        ', resolution: '+  gt_res(res(resx),/string)+' : ' + strtrim(nnssl,2)]
       box_message,mess
  endfor
endfor

; ------------- eliminate inital value -----------
count=0
if n_elements(ssl) gt 1 then begin
  ssl=ssl(1:*)
  if n_elements(ssm) gt 1 then ssm=ssm(1:*)
  sss=sss(1:*)
  count=n_elements(ssl)
endif
; ------------- time sort all output  ------------
order=0
if n_elements(ssl) gt 1 then order=sort_index(roadmap(ssl),/ss)
ssl=ssl(order)
if triplet then ssm=ssm(order)
sss=sss(order)
if not keyword_set(triplet) then ssm=sss
;stop

end
